import pytest

hardware = "rpi"
classname = "adi.adis16495"


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize(
    "classname, attr, max_pow, tol", [(classname, "sample_rate", 11, 0)],
)
def test_adis16495_sample_rate(
    test_attribute_single_value_pow2, iio_uri, classname, attr, max_pow, tol
):
    test_attribute_single_value_pow2(iio_uri, classname, attr, max_pow, tol)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("anglvel_x_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("anglvel_y_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("anglvel_z_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("accel_x_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("accel_y_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("accel_z_filter_low_pass_3db_frequency", 0, 300, 1, 200),
        ("anglvel_x_calibscale", -32768, 32768, 1, 0),
        ("anglvel_y_calibscale", -32768, 32768, 1, 0),
        ("anglvel_z_calibscale", -32768, 32768, 1, 0),
        ("accel_x_calibscale", -32768, 32768, 1, 0),
        ("accel_y_calibscale", -32768, 32768, 1, 0),
        ("accel_z_calibscale", -32768, 32768, 1, 0),
        ("anglvel_x_calibbias", -2147483648, 2147483647, 1, 0),
        ("anglvel_y_calibbias", -2147483648, 2147483647, 1, 0),
        ("anglvel_z_calibbias", -2147483648, 2147483647, 1, 0),
        ("accel_x_calibbias", -2147483648, 2147483647, 1, 0),
        ("accel_y_calibbias", -2147483648, 2147483647, 1, 0),
        ("accel_z_calibbias", -2147483648, 2147483647, 1, 0),
    ],
)
def test_adis16495_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


@pytest.mark.skip(reason="Test is currently failing...")
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3, 4, 5])
def test_adis16495_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



import pytest

hardware = ["fmcomms4", "adrv9364", "pluto", "adrv9361"]
classname = "adi.ad9364"


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("tx_hardwaregain_chan0", -89.75, 0.0, 0.25, 0),
        ("rx_lo", 70000000, 6000000000, 1, 8),
        ("tx_lo", 47000000, 6000000000, 1, 8),
        ("sample_rate", 2084000, 61440000, 1, 4),
    ],
)
def test_ad9364_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr, tol", [("loopback", 0)])
@pytest.mark.parametrize("val", [0, 1, 2])
def test_ad9364_loopback_attr(
    test_attribute_single_value_str, iio_uri, classname, attr, val, tol
):
    test_attribute_single_value_str(iio_uri, classname, attr, val, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_ad9364_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_ad9364_tx_data(test_dma_tx, iio_uri, classname, channel):
    test_dma_tx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=1000000000,
            rx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        )
    ],
)
def test_ad9364_cyclic_buffers(
    test_cyclic_buffer, iio_uri, classname, channel, param_set
):
    test_cyclic_buffer(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=1000000000,
            rx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        )
    ],
)
def test_ad9364_cyclic_buffers_exception(
    test_cyclic_buffer_exception, iio_uri, classname, channel, param_set
):
    test_cyclic_buffer_exception(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_ad9364_loopback(test_dma_loopback, iio_uri, classname, channel):
    test_dma_loopback(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=1000000000,
            rx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        )
    ],
)
@pytest.mark.parametrize("sfdr_min", [40])
def test_ad9364_sfdr(test_sfdr, iio_uri, classname, channel, param_set, sfdr_min):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize("frequency, scale", [(1000000, 1)])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=1000000000,
            rx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-30,
            sample_rate=4000000,
        )
    ],
)
@pytest.mark.parametrize("peak_min", [-40])
def test_ad9364_dds_loopback(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=1000000000,
            rx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        ),
        dict(
            tx_lo=2000000000,
            rx_lo=2000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        ),
        dict(
            tx_lo=3000000000,
            rx_lo=3000000000,
            gain_control_mode_chan0="slow_attack",
            tx_hardwaregain_chan0=-20,
            sample_rate=4000000,
        ),
    ],
)
def test_ad9364_iq_loopback(test_iq_loopback, iio_uri, classname, channel, param_set):
    test_iq_loopback(iio_uri, classname, channel, param_set)



import pytest

hardware = ["adrv9361", "fmcomms2"]
classname = "adi.ad9361"


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
@pytest.mark.parametrize("repeats", [100])
def test_ad9361_stress_context_creation(
    test_stress_context_creation, iio_uri, classname, channel, repeats
):
    test_stress_context_creation(iio_uri, classname, channel, repeats)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
@pytest.mark.parametrize("repeats", [100])
def test_ad9361_stress_rx_buffer(
    test_stress_rx_buffer_creation, iio_uri, classname, hardware, channel, repeats
):
    test_stress_rx_buffer_creation(iio_uri, classname, channel, repeats)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
@pytest.mark.parametrize("repeats", [100])
def test_ad9361_stress_tx_buffer(
    test_stress_tx_buffer_creation, iio_uri, classname, channel, repeats
):
    test_stress_tx_buffer_creation(iio_uri, classname, channel, repeats)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
@pytest.mark.parametrize("buffer_sizes", [[x * x for x in range(4, 1024)]])
def test_ad9361_stress_rx_buffer_length(
    test_stress_rx_buffer_length, iio_uri, classname, channel, buffer_sizes
):
    test_stress_rx_buffer_length(iio_uri, classname, channel, buffer_sizes)



from os import listdir
from os.path import dirname, join, realpath

import pytest

hardware = ["adrv9002-rx1tx1", "adrv9002-rx2tx2"]
classname = "adi.adrv9002"
profile_path = dirname(realpath(__file__)) + "/adrv9002_profiles/"
nco_test_profile = profile_path + "lte_10_lvds_nco_api_48_8_7.json"
nco_test_stream = profile_path + "lte_10_lvds_nco_api_48_8_7.stream"
lte_20_lvds_profile = profile_path + "lte_20_lvds_api_48_8_7.json"
lte_20_lvds_stream = profile_path + "lte_20_lvds_api_48_8_7.stream"
lte_40_lvds_profile = profile_path + "lte_40_lvds_api_48_8_7.json"
lte_40_lvds_stream = profile_path + "lte_40_lvds_api_48_8_7.stream"
lte_5_cmos_profile = profile_path + "lte_5_cmos_api_48_8_7.json"
lte_5_cmos_stream = profile_path + "lte_5_cmos_api_48_8_7.stream"

#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("rx_hardwaregain_chan0", 0, 34, 0.5, 0),
        ("rx_hardwaregain_chan1", 0, 34, 0.5, 0),
        ("tx_hardwaregain_chan0", -40, 0.0, 0.05, 0),
        ("tx_hardwaregain_chan1", -40, 0.0, 0.05, 0),
        ("tx0_lo", 30000000, 6000000000, 1, 8),
        ("tx1_lo", 30000000, 6000000000, 1, 8),
        ("rx0_lo", 30000000, 6000000000, 1, 8),
        ("rx1_lo", 30000000, 6000000000, 1, 8),
    ],
)
def test_adrv9002_float_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, val",
    [
        ("agc_tracking_en_chan0", [0, 1]),
        ("bbdc_rejection_tracking_en_chan0", [0, 1]),
        ("hd_tracking_en_chan0", [0, 1]),
        ("quadrature_fic_tracking_en_chan0", [0, 1]),
        ("quadrature_w_poly_tracking_en_chan0", [0, 1]),
        ("rfdc_tracking_en_chan0", [0, 1]),
        ("rssi_tracking_en_chan0", [0, 1]),
        ("agc_tracking_en_chan1", [0, 1]),
        ("bbdc_rejection_tracking_en_chan1", [0, 1]),
        ("hd_tracking_en_chan1", [0, 1]),
        ("quadrature_fic_tracking_en_chan1", [0, 1]),
        ("quadrature_w_poly_tracking_en_chan1", [0, 1]),
        ("rfdc_tracking_en_chan1", [0, 1]),
        ("rssi_tracking_en_chan1", [0, 1]),
        ("close_loop_gain_tracking_en_chan0", [0, 1]),
        ("lo_leakage_tracking_en_chan0", [0, 1]),
        ("loopback_delay_tracking_en_chan0", [0, 1]),
        ("pa_correction_tracking_en_chan0", [0, 1]),
        ("quadrature_tracking_en_chan0", [0, 1]),
        ("close_loop_gain_tracking_en_chan1", [0, 1]),
        ("lo_leakage_tracking_en_chan1", [0, 1]),
        ("loopback_delay_tracking_en_chan1", [0, 1]),
        ("pa_correction_tracking_en_chan1", [0, 1]),
        ("quadrature_tracking_en_chan1", [0, 1]),
        ("tx0_en", [0, 1]),
        ("tx1_en", [0, 1]),
        ("rx1_en", [0, 1]),
        ("rx0_en", [0, 1]),
    ],
)
def test_adrv9002_boolean_attr(
    test_attribute_multipe_values, iio_uri, classname, attr, val
):
    test_attribute_multipe_values(iio_uri, classname, attr, val, 0)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, val",
    [
        ("rx0_port_en", ["pin", "spi"]),
        ("rx1_port_en", ["pin", "spi"]),
        ("tx0_port_en", ["pin", "spi"]),
        ("tx1_port_en", ["pin", "spi"]),
        ("tx_ensm_mode_chan0", ["calibrated", "primed", "rf_enabled"]),
        ("tx_ensm_mode_chan1", ["calibrated", "primed", "rf_enabled"]),
        ("rx_ensm_mode_chan0", ["calibrated", "primed", "rf_enabled"]),
        ("rx_ensm_mode_chan1", ["calibrated", "primed", "rf_enabled"]),
        (
            "digital_gain_control_mode_chan0",
            [
                "Gain_Correction_manual_control",
                "Gain_Compensation_manual_control",
                "Gain_Correction_automatic_control",
                "Gain_Compensation_automatic_control",
            ],
        ),
        (
            "digital_gain_control_mode_chan1",
            [
                "Gain_Correction_manual_control",
                "Gain_Compensation_manual_control",
                "Gain_Correction_automatic_control",
                "Gain_Compensation_automatic_control",
            ],
        ),
        ("gain_control_mode_chan0", ["pin", "automatic", "spi"]),
        ("gain_control_mode_chan1", ["pin", "automatic", "spi"]),
        ("atten_control_mode_chan0", ["bypass", "pin", "spi"]),
        ("atten_control_mode_chan1", ["bypass", "pin", "spi"]),
    ],
)
def test_adrv9002_str_attr(
    test_attribute_multipe_values, iio_uri, classname, attr, val
):
    test_attribute_multipe_values(iio_uri, classname, attr, val, 0)


#########################################
# baseband rx sample rate should be > 1MHz to run this test


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, val, depends",
    [
        (
            "interface_gain_chan0",
            ["-36dB", "-30dB", "-24dB", "-18dB", "-12dB", "-6dB", "0dB"],
            dict(
                digital_gain_control_mode_chan0="Gain_Compensation_manual_control",
                rx_ensm_mode_chan0="rf_enabled",
            ),
        ),
        (
            "interface_gain_chan1",
            ["-36dB", "-30dB", "-24dB", "-18dB", "-12dB", "-6dB", "0dB"],
            dict(
                digital_gain_control_mode_chan1="Gain_Compensation_manual_control",
                rx_ensm_mode_chan1="rf_enabled",
            ),
        ),
    ],
)
def test_adrv9002_interface_gain_wideband(
    test_attribute_multipe_values_with_depends, iio_uri, classname, attr, depends, val
):
    test_attribute_multipe_values_with_depends(
        iio_uri, classname, attr, depends, val, 0
    )


#########################################
# baseband rx sample rate should be < 1MHz to run this test


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, val, depends",
    [
        (
            "interface_gain_chan0",
            [
                "-36dB",
                "-30dB",
                "-24dB",
                "-18dB",
                "-12dB",
                "-6dB",
                "0dB",
                "6dB",
                "12dB",
                "18dB",
            ],
            dict(
                digital_gain_control_mode_chan0="Gain_Compensation_manual_control",
                rx_ensm_mode_chan0="rf_enabled",
            ),
        ),
        (
            "interface_gain_chan1",
            [
                "-36dB",
                "-30dB",
                "-24dB",
                "-18dB",
                "-12dB",
                "-6dB",
                "0dB",
                "6dB",
                "12dB",
                "18dB",
            ],
            dict(
                digital_gain_control_mode_chan0="Gain_Compensation_manual_control",
                rx_ensm_mode_chan0="rf_enabled",
            ),
        ),
        (
            "interface_gain_chan0",
            ["0dB", "6dB", "12dB", "18dB"],
            dict(
                digital_gain_control_mode_chan0="Gain_Correction_manual_control",
                rx_ensm_mode_chan0="rf_enabled",
            ),
        ),
        (
            "interface_gain_chan1",
            ["0dB", "6dB", "12dB", "18dB"],
            dict(
                digital_gain_control_mode_chan0="Gain_Correction_manual_control",
                rx_ensm_mode_chan0="rf_enabled",
            ),
        ),
    ],
)
def test_adrv9002_interface_gain_narrowband(
    test_attribute_multipe_values_with_depends, iio_uri, classname, attr, depends, val
):
    test_attribute_multipe_values_with_depends(
        iio_uri, classname, attr, depends, val, 0
    )


#########################################
@pytest.mark.lvds_test
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr", ["profile"])
@pytest.mark.parametrize(
    "profile_file, depends",
    [
        (lte_40_lvds_profile, {"write_stream": lte_40_lvds_stream}),
        (lte_20_lvds_profile, {"write_stream": lte_20_lvds_stream}),
    ],
)
def test_adrv9002_profile_write(
    test_attribute_write_only_str_with_depends,
    iio_uri,
    classname,
    attr,
    profile_file,
    depends,
):
    test_attribute_write_only_str_with_depends(
        iio_uri, classname, attr, profile_file, depends
    )


#########################################
@pytest.mark.lvds_test
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr", ["profile"])
@pytest.mark.parametrize(
    "profile_file, depends", [(nco_test_profile, {"write_stream": nco_test_stream})]
)
def test_adrv9002_nco_write_profile(
    test_attribute_write_only_str_with_depends,
    iio_uri,
    classname,
    attr,
    profile_file,
    depends,
):
    test_attribute_write_only_str_with_depends(
        iio_uri, classname, attr, profile_file, depends
    )


#########################################
@pytest.mark.cmos_test
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr", ["profile"])
@pytest.mark.parametrize(
    "profile_file, depends", [(lte_5_cmos_profile, {"write_stream": lte_5_cmos_stream})]
)
def test_adrv9002_stream_profile_write(
    test_attribute_write_only_str_with_depends,
    iio_uri,
    classname,
    attr,
    profile_file,
    depends,
):
    test_attribute_write_only_str_with_depends(
        iio_uri, classname, attr, profile_file, depends
    )


#########################################
@pytest.mark.cmos_test
@pytest.mark.iio_hardware(hardware)
def test_adrv9002_stream_profile_write_both(iio_uri):
    import adi

    sdr = adi.adrv9002(iio_uri)
    sdr.write_stream_profile(lte_5_cmos_stream, lte_5_cmos_profile)


#########################################
# It depends on test_adrv9002_nco_write_profile to be run first.
# Maybe we should think in adding something like pytest-dependency
@pytest.mark.lvds_test
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("rx0_nco_frequency", -20000, 20000, 1, 0),
        ("rx1_nco_frequency", -20000, 20000, 1, 0),
        ("tx0_nco_frequency", -20000, 20000, 1, 0),
        ("tx1_nco_frequency", -20000, 20000, 1, 0),
    ],
)
def test_adrv9002_nco(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware[0])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_adrv9002_tx_data_rx1tx1(test_dma_tx, iio_uri, classname, channel):
    test_dma_tx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware[1])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize("use_tx2", [True])
def test_adrv9002_tx_data_rx2tx2(test_dma_tx, iio_uri, classname, channel, use_tx2):
    test_dma_tx(iio_uri, classname, channel, use_tx2)


#########################################
@pytest.mark.iio_hardware(hardware[0])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_adrv9002_rx_data_rx1tx1(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware[1])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize("use_rx2", [True])
def test_adrv9002_rx_data_rx2tx2(test_dma_rx, iio_uri, classname, channel, use_rx2):
    test_dma_rx(iio_uri, classname, channel, use_rx2)


########################################
@pytest.mark.iio_hardware(hardware[0])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx0_lo=1000000000,
            rx0_lo=1000000000,
            tx1_lo=1000000000,
            rx1_lo=1000000000,
            rx_ensm_mode_chan0="rf_enabled",
            rx_ensm_mode_chan1="rf_enabled",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            tx_ensm_mode_chan0="rf_enabled",
            tx_ensm_mode_chan1="rf_enabled",
        )
    ],
)
def test_adrv9002_cw_loopback(test_cw_loopback, iio_uri, classname, channel, param_set):
    test_cw_loopback(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware[1])
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx0_lo=1000000000,
            rx0_lo=1000000000,
            tx1_lo=1000000000,
            rx1_lo=1000000000,
            rx_ensm_mode_chan0="rf_enabled",
            rx_ensm_mode_chan1="rf_enabled",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            tx_ensm_mode_chan0="rf_enabled",
            tx_ensm_mode_chan1="rf_enabled",
        )
    ],
)
@pytest.mark.parametrize("use_tx2rx2", [False, True])
def test_adrv9002_cw_loopback_split_dma(
    test_cw_loopback, iio_uri, classname, channel, param_set, use_tx2rx2
):
    test_cw_loopback(iio_uri, classname, channel, param_set, use_tx2rx2, use_tx2rx2)



# Copyright (C) 2020 Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     - Neither the name of Analog Devices, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#     - The use of this software may or may not infringe the patent rights
#       of one or more patent holders.  This license does not release you
#       from the requirement that you obtain separate licenses from these
#       patent holders to use this software.
#     - Use of the software either in source or binary form, must be run
#       on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
# RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adxrs290(rx, context_manager, attribute):
    """ ADI ADXRS290 Gyroscope """

    _device_name = "ADXRS290"
    _rx_data_type = np.int16
    _rx_unbuffered_data = True
    _rx_data_si_type = np.float

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("adxrs290")
        self.anglvel_x = self._channel(self._ctrl, "anglvel_x")
        self.anglvel_y = self._channel(self._ctrl, "anglvel_y")
        self.temp = self._channel(self._ctrl, "temp")
        self._rxadc = self._ctx.find_device("adxrs290")
        self._rx_channel_names = ["anglvel_x", "anglvel_y"]
        rx.__init__(self)

    @property
    def hpf_3db_frequency_available(self):
        """Provides all available high pass filter 3dB frequency settings for the ADXRS290 channels"""
        return self._get_iio_attr_str(
            "anglvel_x", "filter_high_pass_3db_frequency_available", False
        )

    @property
    def hpf_3db_frequency(self):
        """ADXRS290 high pass filter 3dB frequency"""
        # Only need to consider one channel, all others follow
        return float(
            self._get_iio_attr_str("anglvel_x", "filter_high_pass_3db_frequency", False)
        )

    @hpf_3db_frequency.setter
    def hpf_3db_frequency(self, value):
        self._set_iio_attr(
            "anglvel_x", "filter_high_pass_3db_frequency", False, str(float(value))
        )

    @property
    def lpf_3db_frequency_available(self):
        """Provides all available low pass filter 3dB frequency settings for the ADXRS290 channels"""
        return self._get_iio_attr_str(
            "anglvel_x", "filter_low_pass_3db_frequency_available", False
        )

    @property
    def lpf_3db_frequency(self):
        """ADXRS290 low pass filter 3dB frequency"""
        # Only need to consider one channel, all others follow
        return float(
            self._get_iio_attr_str("anglvel_x", "filter_low_pass_3db_frequency", False)
        )

    @lpf_3db_frequency.setter
    def lpf_3db_frequency(self, value):
        self._set_iio_attr(
            "anglvel_x", "filter_low_pass_3db_frequency", False, str(float(value))
        )

    class _channel(attribute):
        """ADXRS290 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADXRS290 channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADXRS290 channel scale(gain)"""
            return float(self._get_iio_attr_str(self.name, "scale", False))



import pytest

hardware = "ad7689"
classname = "adi.ad7689"

#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
def test_ad7689_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



# Copyright (C) 2020 Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     - Neither the name of Analog Devices, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#     - The use of this software may or may not infringe the patent rights
#       of one or more patent holders.  This license does not release you
#       from the requirement that you obtain separate licenses from these
#       patent holders to use this software.
#     - Use of the software either in source or binary form, must be run
#       on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
# RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from math import pi, sin

from adi.attribute import attribute
from adi.context_manager import context_manager


class adf4371(attribute, context_manager):
    """ADF4371 Microwave Wideband Synthesizer
    with Integrated VCO

    parameters:
        uri: type=string
            URI of IIO context with ADF4371
    """

    _device_name = "adf4371"
    _muxout_options = (
        "tristate",
        "digital_lock",
        "charge_pump_up",
        "charge_pump_down",
        "RDIV2",
        "N_div_out",
        "VCO_test",
        "high",
        "VCO_calib_R_band",
        "VCO_calib_N_band",
    )

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)

        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADF4371 device not found")

    @property
    def muxout_mode(self):
        """Get/Set the MUX output mode"""
        return self._get_iio_dev_attr("muxout_mode", self._ctrl)

    @muxout_mode.setter
    def muxout_mode(self, value):
        """Get/Set the MUX output mode"""

        # Check that the value is valid
        if value.lower().strip() not in self._muxout_options:
            raise ValueError(
                f"muxout_mode of \"{value}\" is invalid. Valid options: {', '.join(self._muxout_options)}"
            )

        self._set_iio_dev_attr("muxout_mode", value, self._ctrl)

    @property
    def rf8_enable(self):
        """Get/Set the enable status of the 8GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage0", "powerdown", True, self._ctrl))
        )

    @rf8_enable.setter
    def rf8_enable(self, value):
        """Get/Set the enable status of the 8GHz RF output"""
        self._set_iio_attr("altvoltage0", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf8_frequency(self):
        """Get/Set the frequency of the 8GHz RF output"""
        return self._get_iio_attr("altvoltage0", "frequency", True, self._ctrl)

    @rf8_frequency.setter
    def rf8_frequency(self, value):
        """Get/Set the frequency of the 8GHz RF output"""
        self._set_iio_attr("altvoltage0", "frequency", True, value, self._ctrl)

    @property
    def rfaux8_enable(self):
        """Get/Set the enable status of the Auxiliary 8GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage1", "powerdown", True, self._ctrl))
        )

    @rfaux8_enable.setter
    def rfaux8_enable(self, value):
        """Get/Set the enable status of the Auxiliary 8GHz RF output"""
        self._set_iio_attr("altvoltage1", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rfaux8_frequency(self):
        """Get/Set the frequency of the Auxiliary 8GHz RF output"""
        return self._get_iio_attr("altvoltage1", "frequency", True, self._ctrl)

    @rfaux8_frequency.setter
    def rfaux8_frequency(self, value):
        """Get/Set the frequency of the Auxiliary 8GHz RF output"""
        self._set_iio_attr("altvoltage1", "frequency", True, value, self._ctrl)

    @property
    def rf16_enable(self):
        """Get/Set the enable status of the 16GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage2", "powerdown", True, self._ctrl))
        )

    @rf16_enable.setter
    def rf16_enable(self, value):
        """Get/Set the enable status of the 16GHz RF output"""
        self._set_iio_attr("altvoltage2", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf16_frequency(self):
        """Get/Set the frequency of the 16GHz RF output"""
        return self._get_iio_attr("altvoltage2", "frequency", True, self._ctrl)

    @rf16_frequency.setter
    def rf16_frequency(self, value):
        """Get/Set the frequency of the 16GHz RF output"""
        self._set_iio_attr("altvoltage2", "frequency", True, value, self._ctrl)

    @property
    def rf32_enable(self):
        """Get/Set the enable status of the 32GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage3", "powerdown", True, self._ctrl))
        )

    @rf32_enable.setter
    def rf32_enable(self, value):
        """Get/Set the enable status of the 32GHz RF output"""
        self._set_iio_attr("altvoltage3", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf32_frequency(self):
        """Get/Set the frequency of the 32GHz RF output"""
        return self._get_iio_attr("altvoltage3", "frequency", True, self._ctrl)

    @rf32_frequency.setter
    def rf32_frequency(self, value):
        """Get/Set the frequency of the 32GHz RF output"""
        self._set_iio_attr("altvoltage3", "frequency", True, value, self._ctrl)

    @property
    def temperature(self):
        """Get the temperature reading"""
        return self._get_iio_attr("temp0", "input", False)



# Copyright (C) 2019 Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     - Neither the name of Analog Devices, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#     - The use of this software may or may not infringe the patent rights
#       of one or more patent holders.  This license does not release you
#       from the requirement that you obtain separate licenses from these
#       patent holders to use this software.
#     - Use of the software either in source or binary form, must be run
#       on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
# RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from adi.ad936x import ad9361, ad9363, ad9364, Pluto

from adi.fmcomms5 import FMComms5

from adi.ad9371 import ad9371

from adi.adrv9002 import adrv9002

from adi.adrv9009 import adrv9009

from adi.adrv9009_zu11eg import adrv9009_zu11eg

from adi.adrv9009_zu11eg_multi import adrv9009_zu11eg_multi

from adi.adrv9009_zu11eg_fmcomms8 import adrv9009_zu11eg_fmcomms8

from adi.ad9081 import ad9081

from adi.ad9081_mc import ad9081_mc, QuadMxFE

from adi.ad9083 import ad9083

from adi.ad9094 import ad9094

from adi.ad9680 import ad9680

from adi.ad9136 import ad9136

from adi.ad9144 import ad9144

from adi.ad9152 import ad9152

from adi.cn0532 import cn0532

from adi.daq2 import DAQ2

from adi.daq3 import DAQ3

from adi.adis16460 import adis16460

from adi.adis16495 import adis16495

from adi.adis16507 import adis16507

from adi.ad7124 import ad7124

from adi.adxl345 import adxl345

from adi.adxrs290 import adxrs290

from adi.fmclidar1 import fmclidar1

from adi.ad5686 import ad5686

from adi.adar1000 import adar1000, adar1000_array

from adi.ltc2983 import ltc2983

from adi.one_bit_adc_dac import one_bit_adc_dac

from adi.ltc2314_14 import ltc2314_14

from adi.ad7606 import ad7606

from adi.ad7799 import ad7799

from adi.ad7746 import ad7746

from adi.adpd410x import adpd410x

from adi.ad7689 import ad7689

from adi.adf4371 import adf4371

from adi.adpd188 import adpd188

try:
    from adi.jesd import jesd
except ImportError:
    pass

__version__ = "0.0.9"
name = "Analog Devices Hardware Interfaces"



import pytest

hardware = "daq2"
classname = "adi.ad9680"


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
def test_ad9680_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



# Copyright (C) 2019 Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     - Neither the name of Analog Devices, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#     - The use of this software may or may not infringe the patent rights
#       of one or more patent holders.  This license does not release you
#       from the requirement that you obtain separate licenses from these
#       patent holders to use this software.
#     - Use of the software either in source or binary form, must be run
#       on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
# RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from typing import List

import iio

import numpy as np
from adi.attribute import attribute
from adi.dds import dds


class phy(attribute):
    _ctrl: iio.Device = []

    def __del__(self):
        self._ctrl = []


class rx(attribute):
    """Buffer handling for receive devices"""

    _rxadc: iio.Device = []
    _rx_channel_names: List[str] = []
    _complex_data = False
    _rx_data_type = np.int16
    _rx_data_si_type = np.int16
    _rx_mask = 0x0000
    _rx_shift = 0
    __rx_buffer_size = 1024
    __rx_enabled_channels = [0]
    rx_output_type = "raw"
    __rxbuf = None
    _rx_unbuffered_data = False

    def __init__(self, rx_buffer_size=1024):
        if self._complex_data:
            N = 2
        else:
            N = 1
        rx_enabled_channels = list(range(len(self._rx_channel_names) // N))
        self._num_rx_channels = len(self._rx_channel_names)
        self.rx_enabled_channels = rx_enabled_channels
        self.rx_buffer_size = rx_buffer_size

    @property
    def rx_buffer_size(self):
        """rx_buffer_size: Size of receive buffer in samples"""
        return self.__rx_buffer_size

    @rx_buffer_size.setter
    def rx_buffer_size(self, value):
        self.__rx_buffer_size = value

    @property
    def rx_enabled_channels(self):
        """rx_enabled_channels: List of enabled channels (channel 1 is 0)"""
        return self.__rx_enabled_channels

    @rx_enabled_channels.setter
    def rx_enabled_channels(self, value):
        if not value:
            raise Exception("rx_enabled_channels cannot be empty")
        if self._complex_data:
            if max(value) > ((self._num_rx_channels) / 2 - 1):
                raise Exception("RX mapping exceeds available channels")
        else:
            if max(value) > ((self._num_rx_channels) - 1):
                raise Exception("RX mapping exceeds available channels")
        self.__rx_enabled_channels = value

    @property
    def _num_rx_channels_enabled(self):
        return len(self.__rx_enabled_channels)

    def rx_destroy_buffer(self):
        """rx_destroy_buffer: Clears RX buffer"""
        self.__rxbuf = None

    def __del__(self):
        self.__rxbuf = []
        self._rxadc = []

    def _rx_init_channels(self):
        for m in self._rx_channel_names:
            v = self._rxadc.find_channel(m)
            v.enabled = False

        if self._complex_data:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2])
                v.enabled = True
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2 + 1])
                v.enabled = True
        else:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m])
                v.enabled = True
        self.__rxbuf = iio.Buffer(self._rxadc, self.__rx_buffer_size, False)

    def __rx_unbuffered_data(self):
        x = []
        t = self._rx_data_si_type if self.rx_output_type == "SI" else self._rx_data_type
        for _ in range(len(self.rx_enabled_channels)):
            x.append(np.zeros(self.rx_buffer_size, dtype=t))

        # Get scalers first
        if self.rx_output_type == "SI":
            rx_scale = []
            rx_offset = []
            for i in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[i])
                if "scale" in v.attrs:
                    scale = self._get_iio_attr(
                        self._rx_channel_names[i], "scale", False
                    )
                else:
                    scale = 1.0

                if "offset" in v.attrs:
                    offset = self._get_iio_attr(
                        self._rx_channel_names[i], "offset", False
                    )
                else:
                    offset = 0.0
                rx_scale.append(scale)
                rx_offset.append(offset)

        for samp in range(self.rx_buffer_size):
            for i, m in enumerate(self.rx_enabled_channels):
                s = self._get_iio_attr(
                    self._rx_channel_names[m], "raw", False, self._rxadc
                )
                if self.rx_output_type == "SI":
                    x[i][samp] = rx_scale[i] * s + rx_offset[i]
                else:
                    x[i][samp] = s

        return x

    def __rx_complex(self):
        if not self.__rxbuf:
            self._rx_init_channels()
        self.__rxbuf.refill()
        data = self.__rxbuf.read()
        x = np.frombuffer(data, dtype=self._rx_data_type)
        indx = 0
        sig = []
        stride = len(self.rx_enabled_channels) * 2
        for _ in range(stride // 2):
            sig.append(x[indx::stride] + 1j * x[indx + 1 :: stride])
            indx = indx + 2
        # Don't return list if a single channel
        if indx == 2:
            return sig[0]
        return sig

    def __multi_type_rx(self, data):
        """Process buffers with multiple data types"""
        # Process each channel at a time
        channel_bytes = []
        curated_rx_type = []
        for en_ch in self.rx_enabled_channels:
            channel_bytes += [np.dtype(self._rx_data_type[en_ch]).itemsize]
            curated_rx_type += [self._rx_data_type[en_ch]]
        offset = 0
        stride = sum(channel_bytes)
        sig = []
        for indx, chan_bytes in enumerate(channel_bytes):
            bar = bytearray()
            for bytesI in range(offset, len(data), stride):
                bar.extend(data[bytesI : bytesI + chan_bytes])

            sig.append(np.frombuffer(bar, dtype=curated_rx_type[indx]))
            offset += chan_bytes
        return sig

    def __rx_non_complex(self):
        if not self.__rxbuf:
            self._rx_init_channels()
        self.__rxbuf.refill()
        data = self.__rxbuf.read()

        if isinstance(self._rx_data_type, list):
            return self.__multi_type_rx(data)

        x = np.frombuffer(data, dtype=self._rx_data_type)
        if self._rx_mask != 0:
            x = np.bitwise_and(x, self._rx_mask)
        if self._rx_shift > 0:
            x = np.right_shift(x, self._rx_shift)
        elif self._rx_shift < 0:
            x = np.left_shift(x, -(self._rx_shift))

        sig = []
        stride = len(self.rx_enabled_channels)

        if self.rx_output_type == "raw":
            for c in range(stride):
                sig.append(x[c::stride])
        elif self.rx_output_type == "SI":
            rx_scale = []
            rx_offset = []
            for i in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[i])
                if "scale" in v.attrs:
                    scale = self._get_iio_attr(
                        self._rx_channel_names[i], "scale", False
                    )
                else:
                    scale = 1.0

                if "offset" in v.attrs:
                    offset = self._get_iio_attr(
                        self._rx_channel_names[i], "offset", False
                    )
                else:
                    offset = 0.0
                rx_scale.append(scale)
                rx_offset.append(offset)

            for c in range(stride):
                raw = x[c::stride]
                sig.append(raw * rx_scale[c] + rx_offset[c])
        else:
            raise Exception("rx_output_type undefined")

        # Don't return list if a single channel
        if len(self.rx_enabled_channels) == 1:
            return sig[0]
        return sig

    def rx(self):
        """Receive data from hardware buffers for each channel index in
        rx_enabled_channels.

        returns: type=numpy.array or list of numpy.array
            An array or list of arrays when more than one receive channel
            is enabled containing samples from a channel or set of channels.
            Data will be complex when using a complex data device.
        """
        if self._rx_unbuffered_data:
            return self.__rx_unbuffered_data()
        else:
            if self._complex_data:
                return self.__rx_complex()
            else:
                return self.__rx_non_complex()


class tx(dds, attribute):
    """Buffer handling for transmit devices"""

    _tx_buffer_size = 1024
    _txdac: iio.Device = []
    _tx_channel_names: List[str] = []
    _complex_data = False
    __txbuf = None

    def __init__(self, tx_cyclic_buffer=False):
        if self._complex_data:
            N = 2
        else:
            N = 1
        tx_enabled_channels = list(range(len(self._tx_channel_names) // N))
        self._num_tx_channels = len(self._tx_channel_names)
        self.tx_enabled_channels = tx_enabled_channels
        self.tx_cyclic_buffer = tx_cyclic_buffer
        dds.__init__(self)

    def __del__(self):
        self._txdac = []

    @property
    def tx_cyclic_buffer(self):
        """tx_cyclic_buffer: Enable cyclic buffer for TX"""
        return self.__tx_cyclic_buffer

    @tx_cyclic_buffer.setter
    def tx_cyclic_buffer(self, value):
        if self.__txbuf:
            raise Exception(
                "TX buffer already created, buffer must be "
                "destroyed then recreated to modify tx_cyclic_buffer"
            )
        self.__tx_cyclic_buffer = value

    @property
    def _num_tx_channels_enabled(self):
        return len(self.tx_enabled_channels)

    @property
    def tx_enabled_channels(self):
        """tx_enabled_channels: List of enabled channels (channel 1 is 0)"""
        return self.__tx_enabled_channels

    @tx_enabled_channels.setter
    def tx_enabled_channels(self, value):
        if not value:
            self.__tx_enabled_channels = value
            return
        if self._complex_data:
            if max(value) > ((self._num_tx_channels) / 2 - 1):
                raise Exception("TX mapping exceeds available channels")
        else:
            if max(value) > ((self._num_tx_channels) - 1):
                raise Exception("TX mapping exceeds available channels")
        self.__tx_enabled_channels = value

    def tx_destroy_buffer(self):
        """tx_destroy_buffer: Clears TX buffer"""
        self.__txbuf = None

    def _tx_init_channels(self):
        if self._complex_data:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m * 2], True)
                v.enabled = True
                v = self._txdac.find_channel(self._tx_channel_names[m * 2 + 1], True)
                v.enabled = True
        else:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m], True)
                v.enabled = True
        self.__txbuf = iio.Buffer(
            self._txdac, self._tx_buffer_size, self.__tx_cyclic_buffer
        )

    def tx(self, data_np=None):
        """Transmit data to hardware buffers for each channel index in
        tx_enabled_channels.

        args: type=numpy.array or list of numpy.array
            An array or list of arrays when more than one transmit channel
            is enabled containing samples from a channel or set of channels.
            Data must be complex when using a complex data device.
        """
        if not self.__tx_enabled_channels and data_np:
            raise Exception(
                "When tx_enabled_channels is None or empty,"
                + " the input to tx() must be None or empty or not provided"
            )
        if not self.__tx_enabled_channels:
            # Set TX DAC to zero source
            for chan in self._txdac.channels:
                if chan.output:
                    chan.attrs["raw"].value = "0"
                    return
            raise Exception("No DDS channels found for TX, TX zeroing does not apply")

        if self.__txbuf and self.tx_cyclic_buffer:
            raise Exception(
                "TX buffer has been submitted in cyclic mode. "
                "To push more data the tx buffer must be destroyed first."
            )

        if self._complex_data:
            if self._num_tx_channels_enabled == 1:
                data_np = [data_np]

            if len(data_np) != self._num_tx_channels_enabled:
                raise Exception("Not enough data provided for channel mapping")

            indx = 0
            stride = self._num_tx_channels_enabled * 2
            data = np.empty(stride * len(data_np[0]), dtype=np.int16)
            for chan in data_np:
                i = np.real(chan)
                q = np.imag(chan)
                data[indx::stride] = i.astype(int)
                data[indx + 1 :: stride] = q.astype(int)
                indx = indx + 2
        else:
            if self._num_tx_channels_enabled == 1:
                data_np = [data_np]

            if len(data_np) != self._num_tx_channels_enabled:
                raise Exception("Not enough data provided for channel mapping")

            indx = 0
            stride = self._num_tx_channels_enabled
            data = np.empty(stride * len(data_np[0]), dtype=np.int16)
            for chan in data_np:
                data[indx::stride] = chan.astype(int)
                indx = indx + 1

        if not self.__txbuf:
            self.disable_dds()
            self._tx_buffer_size = len(data) // stride
            self._tx_init_channels()

        if len(data) // stride != self._tx_buffer_size:
            raise Exception(
                "Buffer length different than data length. "
                "Cannot change buffer length on the fly"
            )

        # Send data to buffer
        self.__txbuf.write(bytearray(data))
        self.__txbuf.push()


class rx_tx(rx, tx, phy):
    def __init__(self):
        rx.__init__(self)
        tx.__init__(self)

    def __del__(self):
        rx.__del__(self)
        tx.__del__(self)
        phy.__del__(self)



import pytest

hardware = "adrv9002"
classname = ""


#########################################
# fmt: off
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize(
    "attrtype, dev_name, chan_name, inout, attr, start, stop, step, tol, repeats",
    [
        ("channel", "adrv9002-phy", "voltage0", False, "hardwaregain", -89.75, 0.0, 0.25, 0, 100),
        ("channel", "adrv9002-phy", "altvoltage0", True, "RX1_LO_frequency", 70000000, 6000000000, 1, 4, 100),
        ("channel", "adrv9002-phy", "altvoltage1", True, "RX2_LO_frequency", 70000000, 6000000000, 1, 4, 100)
    ],
)
def test_iio_attr(
    test_iio_attribute_single_value,
    iio_uri,
    attrtype,
    dev_name,
    chan_name,
    inout,
    attr,
    start,
    stop,
    step,
    tol,
    repeats,
):
    test_iio_attribute_single_value(
        iio_uri,
        attrtype,
        dev_name,
        chan_name,
        inout,
        attr,
        start,
        stop,
        step,
        tol,
        repeats,
    )
# fmt: on



import pytest

hardware = "ad7799"
classname = "adi.ad7799"


#########################################
@pytest.mark.iio_hardware("ad7799")
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol", [("gain", 0, 7, 1, 0),],
)
def test_ad7799_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, step, stop, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(3))
def test_ad7799_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



import pytest

hardware = "packrf"
classname = "adi.adis16460"


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize(
    "classname, attr, max_pow, tol", [(classname, "sample_rate", 12, 0)],
)
def test_adis16460_sample_rate(
    test_attribute_single_value_pow2, iio_uri, classname, attr, max_pow, tol
):
    test_attribute_single_value_pow2(iio_uri, classname, attr, max_pow, tol)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3, 4, 5])
def test_adis16460_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



import pytest

hardware = "daq2"
classname = "adi.DAQ2"


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
def test_daq2_tx_data(test_dma_tx, iio_uri, classname, channel):
    test_dma_tx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, [0, 1]])
def test_daq2_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize("frequency, scale", [(200000000, 0.5)])
@pytest.mark.parametrize("param_set", [dict()])
@pytest.mark.parametrize("peak_min", [-40])
def test_daq2_dds_loopback(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set", [dict(),],
)
def test_daq2_cw_loopback(test_cw_loopback, iio_uri, classname, channel, param_set):
    test_cw_loopback(iio_uri, classname, channel, param_set)



from os import listdir
from os.path import dirname, join, realpath

import pytest

hardware = "ad9371"
classname = "adi.ad9371"

profile_path = dirname(realpath(__file__)) + "/ad9371_5_profiles/"
test_profiles = [join(profile_path, f) for f in listdir(profile_path)]

params = dict(
    one_cw_tone_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    one_cw_tone_auto=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="automatic",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_attenuation_5dB_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=-5,
        tx_hardwaregain_chan1=-5,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_attenuation_10dB_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_attenuation_0dB_auto=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="automatic",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_attenuation_20dB_auto=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="automatic",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=-20,
        tx_hardwaregain_chan1=-20,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_rf_gain_0dB_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=0,
        rx_hardwaregain_chan1=0,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_rf_gain_20dB_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=20,
        rx_hardwaregain_chan1=20,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=0,
        rx_temp_comp_gain_chan1=0,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_temp_gain_up=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=3,
        rx_temp_comp_gain_chan1=3,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
    change_temp_gain_down=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        rx_lo=2500000000,
        gain_control_mode="manual",
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        rx_temp_comp_gain_chan0=-3,
        rx_temp_comp_gain_chan1=-3,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
    ),
)

params_obs = dict(
    obs_orx1_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    obs_orx1_change_attenuation_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    obs_orx1_change_rf_gain_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=10,
        obs_temp_comp_gain=0,
    ),
    obs_orx1_change_rf_gain_5dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=5,
        obs_temp_comp_gain=0,
    ),
    obs_orx1_change_temp_gain_up=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=3,
    ),
    obs_orx1_change_temp_gain_down=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=-3,
    ),
    obs_orx2_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    obs_orx2_change_attenuation_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    obs_orx2_change_rf_gain_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=10,
        obs_temp_comp_gain=0,
    ),
    obs_orx2_change_rf_gain_5dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=5,
        obs_temp_comp_gain=0,
    ),
    obs_orx2_change_temp_gain_up=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=3,
    ),
    obs_orx2_change_temp_gain_down=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_TX_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=-3,
    ),
    snf_orx1_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    snf_orx1_change_attenuation_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    snf_orx1_change_rf_gain_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=10,
        obs_temp_comp_gain=0,
    ),
    snf_orx1_change_rf_gain_5dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX1_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=5,
        obs_temp_comp_gain=0,
    ),
    snf_orx2_manual=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    snf_orx2_change_attenuation_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=0,
        obs_temp_comp_gain=0,
    ),
    snf_orx2_change_rf_gain_10dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=10,
        obs_temp_comp_gain=0,
    ),
    snf_orx2_change_rf_gain_5dB=dict(
        ensm_mode="radio_on",
        tx_lo=2500000000,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        sn_lo=2500000000,
        obs_quadrature_tracking_en=1,
        obs_rf_port_select="ORX2_SN_LO",
        obs_gain_control_mode="manual",
        obs_hardwaregain=5,
        obs_temp_comp_gain=0,
    ),
)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("tx_hardwaregain_chan0", -41.95, 0.0, 0.05, 0.05),
        ("tx_hardwaregain_chan1", -41.95, 0.0, 0.05, 0.05),
        ("tx_lo", 70000000, 6000000000, 1000, 0),
        ("rx_lo", 70000000, 6000000000, 1000, 0),
    ],
)
def test_ad9371_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(2))
def test_ad9371_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency, scale, peak_min",
    [
        (params["one_cw_tone_manual"], 2000000, 0.5, -13),
        (params["one_cw_tone_manual"], 2000000, 0.12, -25),
        (params["one_cw_tone_manual"], 2000000, 0.25, -19),
        (params["one_cw_tone_auto"], 1000000, 0.12, -14.7),
        (params["one_cw_tone_auto"], 2000000, 0.12, -14.7),
        (params["one_cw_tone_auto"], 500000, 0.12, -14.7),
        (params["change_attenuation_5dB_manual"], 2000000, 0.25, -23.8),
        (params["change_attenuation_10dB_manual"], 2000000, 0.25, -28.75),
        (params["change_attenuation_0dB_auto"], 1000000, 0.12, -9),
        (params["change_attenuation_20dB_auto"], 1000000, 0.12, -24.7),
        (params["change_rf_gain_0dB_manual"], 2000000, 0.25, -29),
        (params["change_rf_gain_20dB_manual"], 2000000, 0.25, -9),
        (params["change_temp_gain_up"], 2000000, 0.25, -16),
        (params["change_temp_gain_down"], 2000000, 0.25, -22),
    ],
)
def test_ad9371_dds_loopback(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency, scale, peak_min",
    [
        (params["one_cw_tone_manual"], 2000000, 0.5, -33),
        (params["one_cw_tone_manual"], 2000000, 0.12, -45),
        (params["one_cw_tone_manual"], 2000000, 0.25, -39),
        (params["one_cw_tone_auto"], 1000000, 0.12, -34.7),
        (params["one_cw_tone_auto"], 2000000, 0.12, -34.7),
        (params["one_cw_tone_auto"], 500000, 0.12, -34.7),
        (params["change_attenuation_5dB_manual"], 2000000, 0.25, -43.8),
        (params["change_attenuation_10dB_manual"], 2000000, 0.25, -48.75),
        (params["change_attenuation_0dB_auto"], 1000000, 0.12, -29),
        (params["change_attenuation_20dB_auto"], 1000000, 0.12, -44.7),
        (params["change_rf_gain_0dB_manual"], 2000000, 0.25, -49),
        (params["change_rf_gain_20dB_manual"], 2000000, 0.25, -29),
        (params["change_temp_gain_up"], 2000000, 0.25, -36),
        (params["change_temp_gain_down"], 2000000, 0.25, -42),
    ],
)
def test_ad9371_dds_loopback_with_10dB_splitter(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "channel, param_set",
    [
        (0, params_obs["obs_orx1_manual"]),
        (0, params_obs["obs_orx1_change_rf_gain_10dB"]),
        (0, params_obs["obs_orx1_change_rf_gain_5dB"]),
        (0, params_obs["obs_orx1_change_temp_gain_up"]),
        (0, params_obs["obs_orx1_change_temp_gain_down"]),
        (1, params_obs["obs_orx2_manual"]),
        (1, params_obs["obs_orx2_change_rf_gain_10dB"]),
        (1, params_obs["obs_orx2_change_rf_gain_5dB"]),
        (1, params_obs["obs_orx2_change_temp_gain_up"]),
        (1, params_obs["obs_orx2_change_temp_gain_down"]),
        (0, params_obs["snf_orx1_manual"]),
        (0, params_obs["snf_orx1_change_rf_gain_10dB"]),
        (0, params_obs["snf_orx1_change_rf_gain_5dB"]),
        (1, params_obs["snf_orx2_manual"]),
        (1, params_obs["snf_orx2_change_rf_gain_10dB"]),
        (1, params_obs["snf_orx2_change_rf_gain_5dB"]),
    ],
)
@pytest.mark.parametrize(
    "frequency, scale, peak_min, use_obs", [(5000000, 0.25, -40.5, True)]
)
def test_ad9371_dds_loopback_for_obs(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
    use_obs,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min, use_obs
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency1, scale1, peak_min1, frequency2, scale2, peak_min2",
    [(params["one_cw_tone_auto"], 1000000, 0.06, -21, 2000000, 0.12, -15)],
)
def test_ad9371_two_tone_loopback(
    test_dds_two_tone,
    iio_uri,
    classname,
    channel,
    param_set,
    frequency1,
    scale1,
    peak_min1,
    frequency2,
    scale2,
    peak_min2,
):
    test_dds_two_tone(
        iio_uri,
        classname,
        channel,
        param_set,
        frequency1,
        scale1,
        peak_min1,
        frequency2,
        scale2,
        peak_min2,
    )


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency1, scale1, peak_min1, frequency2, scale2, peak_min2",
    [(params["one_cw_tone_auto"], 1000000, 0.06, -41, 2000000, 0.12, -35)],
)
def test_ad9371_two_tone_loopback_with_10dB_splitter(
    test_dds_two_tone,
    iio_uri,
    classname,
    channel,
    param_set,
    frequency1,
    scale1,
    peak_min1,
    frequency2,
    scale2,
    peak_min2,
):
    test_dds_two_tone(
        iio_uri,
        classname,
        channel,
        param_set,
        frequency1,
        scale1,
        peak_min1,
        frequency2,
        scale2,
        peak_min2,
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_manual"], 0.5, 8.5, 9.5),
        (params["one_cw_tone_manual"], 0.12, 20.5, 21.5),
        (params["one_cw_tone_manual"], 0.25, 14.5, 15.5),
        (params["one_cw_tone_auto"], 0.12, 10.5, 11.5),
        (params["change_attenuation_5dB_manual"], 0.25, 19.5, 20.5),
        (params["change_attenuation_10dB_manual"], 0.25, 24.25, 25.25),
        (params["change_attenuation_0dB_auto"], 0.12, 3.5, 4.75),
        (params["change_attenuation_20dB_auto"], 0.12, 20.75, 22),
        (params["change_rf_gain_0dB_manual"], 0.25, 24.75, 26),
        (params["change_rf_gain_20dB_manual"], 0.25, 5, 6),
        (params["change_temp_gain_up"], 0.25, 14.5, 15.5),
        (params["change_temp_gain_down"], 0.25, 14.5, 15.5),
    ],
)
def test_ad9371_dds_gain_check_vary_power(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_manual"], 0.5, 30, 31),
        (params["one_cw_tone_manual"], 0.12, 42.5, 43.5),
        (params["one_cw_tone_manual"], 0.25, 35.5, 36.5),
        (params["one_cw_tone_auto"], 0.12, 32.5, 33.5),
        (params["change_attenuation_5dB_manual"], 0.25, 41, 42),
        (params["change_attenuation_10dB_manual"], 0.25, 44, 45),
        (params["change_attenuation_0dB_auto"], 0.12, 22.75, 23.75),
        (params["change_attenuation_20dB_auto"], 0.12, 42, 43),
        (params["change_rf_gain_0dB_manual"], 0.25, 45.5, 46.5),
        (params["change_rf_gain_20dB_manual"], 0.25, 26, 27),
        (params["change_temp_gain_up"], 0.25, 35.75, 36.75),
        (params["change_temp_gain_down"], 0.25, 35.75, 36.75),
    ],
)
def test_ad9371_dds_gain_check_vary_power_with_10dB_splitter(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        params["one_cw_tone_manual"],
        params["one_cw_tone_auto"],
        params["change_attenuation_5dB_manual"],
        params["change_attenuation_10dB_manual"],
        params["change_attenuation_0dB_auto"],
        params["change_attenuation_20dB_auto"],
        params["change_rf_gain_0dB_manual"],
        params["change_rf_gain_20dB_manual"],
        params["change_temp_gain_up"],
        params["change_temp_gain_down"],
    ],
)
@pytest.mark.parametrize("sfdr_min", [45])
def test_ad9371_sfdr(test_sfdr, iio_uri, classname, channel, param_set, sfdr_min):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        params["one_cw_tone_manual"],
        params["one_cw_tone_auto"],
        params["change_attenuation_5dB_manual"],
        params["change_attenuation_10dB_manual"],
        params["change_attenuation_0dB_auto"],
        params["change_attenuation_20dB_auto"],
        params["change_rf_gain_0dB_manual"],
        params["change_rf_gain_20dB_manual"],
        params["change_temp_gain_up"],
        params["change_temp_gain_down"],
    ],
)
@pytest.mark.parametrize("sfdr_min", [45])
def test_ad9371_sfdr_with_10dB_splitter(
    test_sfdr, iio_uri, classname, channel, param_set, sfdr_min
):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "channel, param_set, sfdr_min",
    [
        (0, params_obs["obs_orx1_change_attenuation_10dB"], 45),
        (0, params_obs["snf_orx1_change_attenuation_10dB"], 30),
        (1, params_obs["obs_orx2_change_attenuation_10dB"], 45),
        (1, params_obs["snf_orx2_change_attenuation_10dB"], 30),
    ],
)
@pytest.mark.parametrize("use_obs", [True])
def test_ad9371_sfdr_for_obs(
    test_sfdr, iio_uri, classname, channel, param_set, sfdr_min, use_obs
):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min, use_obs)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr", ["profile"])
@pytest.mark.parametrize(
    "files", test_profiles,
)
def test_ad9371_profile_write(
    test_attribute_write_only_str, iio_uri, classname, attr, files
):
    test_attribute_write_only_str(iio_uri, classname, attr, files)



from os import listdir
from os.path import dirname, join, realpath

import pytest

hardware = "ad9081"
classname = "adi.ad9081"

#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, val",
    [
        ("rx_nyquist_zone", ["even", "odd"]),
        ("loopback_mode", [2, 1, 0]),
        (
            "rx_test_mode",
            [
                "midscale_short",
                "pos_fullscale",
                "neg_fullscale",
                "checkerboard",
                "pn23",
                "pn9",
                "one_zero_toggle",
                "user",
                "pn7",
                "pn15",
                "pn31",
                "ramp",
                "off",
            ],
        ),
        (
            "tx_main_ffh_mode",
            ["phase_continuous", "phase_incontinuous", "phase_coherent"],
        ),
    ],
)
def test_ad9081_str_attr(test_attribute_multipe_values, iio_uri, classname, attr, val):
    test_attribute_multipe_values(iio_uri, classname, attr, val, 0)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol, repeats",
    [
        ("rx_main_nco_frequencies", -2000000000, 2000000000, 1, 1, 10),
        ("tx_main_nco_frequencies", -6000000000, 6000000000, 1, 1, 10),
        ("rx_channel_nco_frequencies", -500000000, 500000000, 1, 1, 10),
        ("tx_channel_nco_frequencies", -750000000, 750000000, 1, 1, 10),
        ("rx_main_nco_phases", -180000, 180000, 1, 1, 10),
        ("tx_main_nco_phases", -180000, 180000, 1, 1, 10),
        ("rx_channel_nco_phases", -180000, 180000, 1, 1, 10),
        ("tx_channel_nco_phases", -180000, 180000, 1, 1, 10),
        ("tx_main_nco_test_tone_scales", 0.0, 1.0, 0.01, 0.01, 10),
        ("tx_channel_nco_test_tone_scales", 0.0, 1.0, 0.01, 0.01, 10),
        ("tx_main_ffh_index", 1, 31, 1, 0, 10),
        ("tx_main_ffh_frequency", -6000000000, 6000000000, 1, 1, 10),
        ("tx_channel_nco_gain_scales", 0.0, 0.5, 0.01, 0.01, 10),
    ],
)
def test_ad9081_attr(
    test_attribute_single_value,
    iio_uri,
    classname,
    attr,
    start,
    stop,
    step,
    tol,
    repeats,
):
    test_attribute_single_value(
        iio_uri, classname, attr, start, stop, step, tol, repeats
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
def test_ad9081_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
def test_ad9081_tx_data(test_dma_tx, iio_uri, classname, channel):
    test_dma_tx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_nyquist_zone="odd",
            tx_channel_nco_gain_scales=[0.5, 0.5, 0.5, 0.5],
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_test_tone_en=[0, 0, 0, 0],
            tx_main_nco_test_tone_en=[0, 0, 0, 0],
        )
    ],
)
def test_ad9081_cyclic_buffers(
    test_cyclic_buffer, iio_uri, classname, channel, param_set
):
    test_cyclic_buffer(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_test_tone_en=[0, 0, 0, 0],
            tx_main_nco_test_tone_en=[0, 0, 0, 0],
        )
    ],
)
def test_ad9081_cyclic_buffers_exception(
    test_cyclic_buffer_exception, iio_uri, classname, channel, param_set
):
    test_cyclic_buffer_exception(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
def test_ad9081_loopback(test_dma_loopback, iio_uri, classname, channel):
    test_dma_loopback(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_test_tone_en=[0, 0, 0, 0],
            tx_main_nco_test_tone_en=[0, 0, 0, 0],
        )
    ],
)
@pytest.mark.parametrize("sfdr_min", [70])
def test_ad9081_sfdr(test_sfdr, iio_uri, classname, channel, param_set, sfdr_min):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min, full_scale=0.5)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize("frequency, scale", [(10000000, 0.5)])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
        )
    ],
)
@pytest.mark.parametrize("peak_min", [-30])
def test_ad9081_dds_loopback(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[500000000, 500000000, 500000000, 500000000],
            tx_main_nco_frequencies=[500000000, 500000000, 500000000, 500000000],
            rx_channel_nco_frequencies=[1234567, 1234567, 1234567, 1234567],
            tx_channel_nco_frequencies=[1234567, 1234567, 1234567, 1234567],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
        ),
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[750000000, 750000000, 750000000, 750000000],
            tx_main_nco_frequencies=[750000000, 750000000, 750000000, 750000000],
            rx_channel_nco_frequencies=[-1234567, -1234567, -1234567, -1234567],
            tx_channel_nco_frequencies=[-1234567, -1234567, -1234567, -1234567],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
        ),
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            rx_main_nco_phases=[0, 0, 0, 0],
            tx_main_nco_phases=[0, 0, 0, 0],
            rx_channel_nco_phases=[0, 0, 0, 0],
            tx_channel_nco_phases=[0, 0, 0, 0],
        ),
    ],
)
def test_ad9081_iq_loopback(test_iq_loopback, iio_uri, classname, channel, param_set):
    test_iq_loopback(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize("frequency", [10000000])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1010000000, 1010000000, 1010000000, 1010000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_main_nco_test_tone_scales=[0.5, 0.5, 0.5, 0.5],
            tx_main_nco_test_tone_en=[1, 1, 1, 1],
            tx_channel_nco_test_tone_en=[0, 0, 0, 0],
        ),
        dict(
            loopback_mode=0,
            rx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            tx_main_nco_frequencies=[1000000000, 1000000000, 1000000000, 1000000000],
            rx_channel_nco_frequencies=[0, 0, 0, 0],
            tx_channel_nco_frequencies=[10000000, 10000000, 10000000, 10000000],
            tx_channel_nco_test_tone_scales=[0.5, 0.5, 0.5, 0.5],
            tx_main_nco_test_tone_en=[0, 0, 0, 0],
            tx_channel_nco_test_tone_en=[1, 1, 1, 1],
        ),
    ],
)
@pytest.mark.parametrize("peak_min", [-30])
def test_ad9081_nco_loopback(
    test_tone_loopback, iio_uri, classname, param_set, channel, frequency, peak_min,
):
    test_tone_loopback(iio_uri, classname, param_set, channel, frequency, peak_min)


#########################################



from os import listdir
from os.path import dirname, join, realpath

import pytest

try:
    import adi.jesd

    skip_jesd = False
except:
    skip_jesd = True


hardware = "adrv9009"
classname = "adi.adrv9009"

profile_path = dirname(realpath(__file__)) + "/adrv9009_profiles/"
test_profiles = [join(profile_path, f) for f in listdir(profile_path)]

params = dict(
    one_cw_tone_manual=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        gain_control_mode_chan0="manual",
        gain_control_mode_chan1="manual",
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    one_cw_tone_slow_attack=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_attenuation_5dB_manual=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        gain_control_mode_chan0="manual",
        gain_control_mode_chan1="manual",
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=-5,
        tx_hardwaregain_chan1=-5,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_attenuation_10dB_manual=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        gain_control_mode_chan0="manual",
        gain_control_mode_chan1="manual",
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        rx_hardwaregain_chan0=10,
        rx_hardwaregain_chan1=10,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_attenuation_0dB_slow_attack=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_attenuation_20dB_slow_attack=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=-20,
        tx_hardwaregain_chan1=-20,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_rf_gain_0dB_manual=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        gain_control_mode_chan0="manual",
        gain_control_mode_chan1="manual",
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        rx_hardwaregain_chan0=0,
        rx_hardwaregain_chan1=0,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_rf_gain_20dB_manual=dict(
        ensm_mode="radio_on",
        trx_lo=2500000000,
        gain_control_mode_chan0="manual",
        gain_control_mode_chan1="manual",
        obs_powerdown_en=1,
        rx_powerdown_en_chan0=0,
        rx_powerdown_en_chan1=0,
        rx_hardwaregain_chan0=20,
        rx_hardwaregain_chan1=20,
        rx_quadrature_tracking_en_chan0=1,
        rx_quadrature_tracking_en_chan1=1,
        tx_hardwaregain_chan0=0,
        tx_hardwaregain_chan1=0,
        tx_quadrature_tracking_en_chan0=1,
        tx_quadrature_tracking_en_chan1=1,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_trx_lo_1GHz_slow_attack=dict(
        trx_lo=1000000000,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_trx_lo_3GHz_slow_attack=dict(
        trx_lo=3000000000,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
    change_trx_lo_5GHz_slow_attack=dict(
        trx_lo=5000000000,
        gain_control_mode_chan0="slow_attack",
        gain_control_mode_chan1="slow_attack",
        tx_hardwaregain_chan0=-10,
        tx_hardwaregain_chan1=-10,
        calibrate_rx_qec_en=1,
        calibrate_tx_qec_en=1,
        calibrate=1,
    ),
)

# params_obs = dict(
#     obs_tx=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_TX_LO",
#         obs_hardwaregain=0,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_tx_change_attenuation_10dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=-10,
#         tx_hardwaregain_chan1=-10,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_TX_LO",
#         obs_hardwaregain=0,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_tx_change_rf_gain_10dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_TX_LO",
#         obs_hardwaregain=10,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_tx_change_rf_gain_5dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_TX_LO",
#         obs_hardwaregain=5,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_aux=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_AUX_LO",
#         obs_hardwaregain=0,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_aux_change_attenuation_10dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=-10,
#         tx_hardwaregain_chan1=-10,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_AUX_LO",
#         obs_hardwaregain=0,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_aux_change_rf_gain_10dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_AUX_LO",
#         obs_hardwaregain=10,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
#     obs_aux_change_rf_gain_5dB=dict(
#         ensm_mode="radio_on",
#         trx_lo=2500000000,
#         rx_powerdown_en_chan0=1,
#         rx_powerdown_en_chan1=1,
#         obs_powerdown_en=0,
#         tx_hardwaregain_chan0=0,
#         tx_hardwaregain_chan1=0,
#         tx_quadrature_tracking_en_chan0=1,
#         tx_quadrature_tracking_en_chan1=1,
#         aux_obs_lo=2500000000,
#         obs_quadrature_tracking_en=1,
#         obs_rf_port_select="OBS_AUX_LO",
#         obs_hardwaregain=5,
#         calibrate_rx_qec_en=1,
#         calibrate_tx_qec_en=1,
#         calibrate=1,
#     ),
# )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("tx_hardwaregain_chan0", -41.95, 0.0, 0.05, 0.05),
        ("tx_hardwaregain_chan1", -41.95, 0.0, 0.05, 0.05),
        ("trx_lo", 70000000, 6000000000, 1000, 0),
    ],
)
def test_adrv9009_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(2))
def test_adrv9009_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency, scale, peak_min",
    [
        (params["one_cw_tone_manual"], 2000000, 0.5, -42),
        (params["one_cw_tone_manual"], 2000000, 0.12, -55),
        (params["one_cw_tone_manual"], 2000000, 0.25, -48),
        (params["one_cw_tone_slow_attack"], 1000000, 0.12, -44),
        (params["one_cw_tone_slow_attack"], 2000000, 0.12, -44),
        (params["one_cw_tone_slow_attack"], 500000, 0.12, -44),
        (params["change_attenuation_5dB_manual"], 2000000, 0.25, -53),
        (params["change_attenuation_10dB_manual"], 2000000, 0.25, -58),
        (params["change_attenuation_0dB_slow_attack"], 1000000, 0.12, -35),
        (params["change_attenuation_20dB_slow_attack"], 1000000, 0.12, -54),
        (params["change_rf_gain_0dB_manual"], 2000000, 0.25, -59),
        (params["change_rf_gain_20dB_manual"], 2000000, 0.25, -38),
    ],
)
def test_adrv9009_dds_loopback(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency, scale, peak_min",
    [
        (params["one_cw_tone_manual"], 2000000, 0.5, -53),
        (params["one_cw_tone_manual"], 2000000, 0.12, -66),
        (params["one_cw_tone_manual"], 2000000, 0.25, -60),
        (params["one_cw_tone_slow_attack"], 1000000, 0.12, -55.5),
        (params["one_cw_tone_slow_attack"], 2000000, 0.12, -55.5),
        (params["one_cw_tone_slow_attack"], 500000, 0.12, -56),
        (params["change_attenuation_5dB_manual"], 2000000, 0.25, -65),
        (params["change_attenuation_10dB_manual"], 2000000, 0.25, -69),
        (params["change_attenuation_0dB_slow_attack"], 1000000, 0.12, -46),
        (params["change_attenuation_20dB_slow_attack"], 1000000, 0.12, -65),
        (params["change_rf_gain_0dB_manual"], 2000000, 0.25, -70),
        (params["change_rf_gain_20dB_manual"], 2000000, 0.25, -50),
    ],
)
def test_adrv9009_dds_loopback_with_10dB_splitter(
    test_dds_loopback,
    iio_uri,
    classname,
    param_set,
    channel,
    frequency,
    scale,
    peak_min,
):
    test_dds_loopback(
        iio_uri, classname, param_set, channel, frequency, scale, peak_min
    )


########################################
# @pytest.mark.obs_required
# @pytest.mark.iio_hardware(hardware)
# @pytest.mark.parametrize("classname", [(classname)])
# @pytest.mark.parametrize(
#     "channel, param_set",
#     [
#         (0, params_obs["obs_tx"]),
#         (0, params_obs["obs_tx_change_attenuation_10dB"]),
#         (0, params_obs["obs_tx_change_rf_gain_10dB"]),
#         (0, params_obs["obs_tx_change_rf_gain_5dB"]),
#         (0, params_obs["obs_aux"]),
#         (0, params_obs["obs_aux_change_attenuation_10dB"]),
#         (0, params_obs["obs_aux_change_rf_gain_10dB"]),
#         (0, params_obs["obs_aux_change_rf_gain_5dB"]),
#     ],
# )
# @pytest.mark.parametrize(
#     "frequency, scale, peak_min, use_obs", [(50000000, 0.25, -40.5, True)]
# )
# def test_adrv9009_dds_loopback_for_obs(
#     test_dds_loopback,
#     iio_uri,
#     classname,
#     param_set,
#     channel,
#     frequency,
#     scale,
#     peak_min,
#     use_obs,
# ):
#     test_dds_loopback(
#         iio_uri, classname, param_set, channel, frequency, scale, peak_min, use_obs
#     )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency1, scale1, peak_min1, frequency2, scale2, peak_min2",
    [(params["one_cw_tone_slow_attack"], 1000000, 0.06, -21, 2000000, 0.12, -15)],
)
def test_adrv9009_two_tone_loopback(
    test_dds_two_tone,
    iio_uri,
    classname,
    channel,
    param_set,
    frequency1,
    scale1,
    peak_min1,
    frequency2,
    scale2,
    peak_min2,
):
    test_dds_two_tone(
        iio_uri,
        classname,
        channel,
        param_set,
        frequency1,
        scale1,
        peak_min1,
        frequency2,
        scale2,
        peak_min2,
    )


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, frequency1, scale1, peak_min1, frequency2, scale2, peak_min2",
    [(params["one_cw_tone_slow_attack"], 1000000, 0.06, -41, 2000000, 0.12, -35)],
)
def test_adrv9009_two_tone_loopback_with_10dB_splitter(
    test_dds_two_tone,
    iio_uri,
    classname,
    channel,
    param_set,
    frequency1,
    scale1,
    peak_min1,
    frequency2,
    scale2,
    peak_min2,
):
    test_dds_two_tone(
        iio_uri,
        classname,
        channel,
        param_set,
        frequency1,
        scale1,
        peak_min1,
        frequency2,
        scale2,
        peak_min2,
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        params["one_cw_tone_manual"],
        params["one_cw_tone_slow_attack"],
        params["change_attenuation_5dB_manual"],
        params["change_attenuation_10dB_manual"],
        params["change_attenuation_0dB_slow_attack"],
        params["change_attenuation_20dB_slow_attack"],
        params["change_rf_gain_0dB_manual"],
        params["change_rf_gain_20dB_manual"],
        params["change_trx_lo_1GHz_slow_attack"],
        params["change_trx_lo_3GHz_slow_attack"],
        params["change_trx_lo_5GHz_slow_attack"],
    ],
)
@pytest.mark.parametrize("sfdr_min", [45])
def test_adrv9009_sfdr(test_sfdr, iio_uri, classname, channel, param_set, sfdr_min):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        params["one_cw_tone_manual"],
        params["one_cw_tone_slow_attack"],
        params["change_attenuation_5dB_manual"],
        params["change_attenuation_10dB_manual"],
        params["change_attenuation_0dB_slow_attack"],
        params["change_attenuation_20dB_slow_attack"],
        params["change_rf_gain_0dB_manual"],
        params["change_rf_gain_20dB_manual"],
        params["change_trx_lo_1GHz_slow_attack"],
        params["change_trx_lo_3GHz_slow_attack"],
        params["change_trx_lo_5GHz_slow_attack"],
    ],
)
@pytest.mark.parametrize("sfdr_min", [40])
def test_adrv9009_sfdr_with_10dB_splitter(
    test_sfdr, iio_uri, classname, channel, param_set, sfdr_min
):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
# @pytest.mark.obs_required
# @pytest.mark.iio_hardware(hardware)
# @pytest.mark.parametrize("classname", [(classname)])
# @pytest.mark.parametrize("channel", [0, 1])
# @pytest.mark.parametrize(
#     "param_set",
#     [
#         params_obs["obs_tx"],
#         params_obs["obs_tx_change_attenuation_10dB"],
#         params_obs["obs_tx_change_rf_gain_10dB"],
#         params_obs["obs_tx_change_rf_gain_5dB"],
#         params_obs["obs_aux"],
#         params_obs["obs_aux_change_attenuation_10dB"],
#         params_obs["obs_aux_change_rf_gain_10dB"],
#         params_obs["obs_aux_change_rf_gain_5dB"],
#     ],
# )
# @pytest.mark.parametrize("sfdr_min", [45])
# @pytest.mark.parametrize("use_obs", [True])
# def test_adrv9009_sfdr_for_obs(
#     test_sfdr, iio_uri, classname, channel, param_set, sfdr_min, use_obs
# ):
#     test_sfdr(iio_uri, classname, channel, param_set, sfdr_min, use_obs)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_slow_attack"], 0.12, 33.5, 41.5),
        (params["change_attenuation_0dB_slow_attack"], 0.12, 24, 32),
        (params["change_attenuation_20dB_slow_attack"], 0.12, 40, 45.5),
        (params["change_trx_lo_1GHz_slow_attack"], 0, 35, 60),
        (params["change_trx_lo_1GHz_slow_attack"], 0.9, 25, 35),
        (params["change_trx_lo_3GHz_slow_attack"], 0, 35, 60),
        (params["change_trx_lo_3GHz_slow_attack"], 0.9, 0, 22),
        (params["change_trx_lo_5GHz_slow_attack"], 0, 35, 60),
        (params["change_trx_lo_5GHz_slow_attack"], 0.9, 20, 25),
    ],
)
def test_adrv9009_dds_gain_check_agc(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_slow_attack"], 0.12, 40, 45),
        (params["change_attenuation_0dB_slow_attack"], 0.12, 30, 42),
        (params["change_attenuation_20dB_slow_attack"], 0.12, 43.5, 45),
        (params["change_trx_lo_1GHz_slow_attack"], 0, 43.5, 45.25),
        (params["change_trx_lo_1GHz_slow_attack"], 0.9, 30, 40),
        (params["change_trx_lo_3GHz_slow_attack"], 0, 43.75, 45),
        (params["change_trx_lo_3GHz_slow_attack"], 0.9, 23, 33),
        (params["change_trx_lo_5GHz_slow_attack"], 0, 43.75, 45),
        (params["change_trx_lo_5GHz_slow_attack"], 0.9, 25, 33),
    ],
)
def test_adrv9009_dds_gain_check_agc_with_10db_splitter(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_manual"], 0.5, 30, 40),
        (params["one_cw_tone_manual"], 0.12, 40, 45),
        (params["one_cw_tone_manual"], 0.25, 35, 45),
        (params["change_attenuation_5dB_manual"], 0.25, 40, 45),
        (params["change_attenuation_10dB_manual"], 0.25, 40, 45),
        (params["change_rf_gain_0dB_manual"], 0.25, 40, 45),
        (params["change_rf_gain_20dB_manual"], 0.25, 25, 35),
        (
            dict(
                trx_lo=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                rx_hardwaregain_chan0=0,
                rx_hardwaregain_chan1=0,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                calibrate_tx_qec_en=1,
                calibrate=1,
            ),
            0.5,
            20,
            60,
        ),
        (
            dict(
                trx_lo=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                rx_hardwaregain_chan0=30,
                rx_hardwaregain_chan1=30,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                calibrate_tx_qec_en=1,
                calibrate=1,
            ),
            0.5,
            30,
            45,
        ),
    ],
)
def test_adrv9009_dds_gain_check_vary_power(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.obs_required
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (params["one_cw_tone_manual"], 0.5, 38, 45),
        (params["one_cw_tone_manual"], 0.12, 43, 45),
        (params["one_cw_tone_manual"], 0.25, 43, 45),
        (params["change_attenuation_5dB_manual"], 0.25, 43, 45),
        (params["change_attenuation_10dB_manual"], 0.25, 43, 45),
        (params["change_rf_gain_0dB_manual"], 0.25, 42.5, 44.5),
        (params["change_rf_gain_20dB_manual"], 0.25, 34, 44),
        (
            dict(
                trx_lo=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                rx_hardwaregain_chan0=0,
                rx_hardwaregain_chan1=0,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                calibrate_tx_qec_en=1,
                calibrate=1,
            ),
            0.5,
            42.5,
            44.5,
        ),
        (
            dict(
                trx_lo=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                rx_hardwaregain_chan0=30,
                rx_hardwaregain_chan1=30,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                calibrate_tx_qec_en=1,
                calibrate=1,
            ),
            0.5,
            36,
            43.5,
        ),
    ],
)
def test_adrv9009_dds_gain_check_vary_power_with_10dB_splitter(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
# TODO: Test, see what's its difference with dds_loopback, PROBLEMATIC
# OSError: [Errno 5] Input/output error
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            trx_lo=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            calibrate_tx_qec_en=1,
            calibrate=1,
        ),
        dict(
            trx_lo=3000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            calibrate_tx_qec_en=1,
            calibrate=1,
        ),
        dict(
            trx_lo=5000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            calibrate_tx_qec_en=1,
            calibrate=1,
        ),
    ],
)
def test_adrv9009_iq_loopback(test_iq_loopback, iio_uri, classname, channel, param_set):
    test_iq_loopback(iio_uri, classname, channel, param_set)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("attr", ["profile"])
@pytest.mark.parametrize(
    "files", test_profiles,
)
def test_adrv9009_profile_write(
    test_attribute_write_only_str, iio_uri, classname, attr, files
):
    test_attribute_write_only_str(iio_uri, classname, attr, files)


#########################################
@pytest.mark.skipif(skip_jesd, reason="JESD module not importable")
@pytest.mark.iio_hardware(hardware)
def test_adrv9009_jesd(iio_uri):
    import adi

    sdr = adi.adrv9009(uri=iio_uri, jesd_monitor=True)
    info = sdr._jesd.get_all_statuses()
    assert info
